MICOV_DATA = {
    "org.micov.medical.1": {
        "last_name": "Rossi",
        "given_name": "Mario",
        "birth_date": "1922-03-13",
        "PersonId_nic": {
            "PersonIdNumber": "1234567890",
            "PersonIdType": "nic",
            "PersonIdIS": "IT",
        },
        "sex": 1,
        "VPInfo_COVID-19_1": {
            "VaccineProphylaxis": "",
            "VaccMedicinalProd": "Moderna",
            "VaccMktAuthHolder": "Moderna",
            "VaccDoseNumber": "2/2",
            "VaccAdmDate": "2021-01-01",
            "VaccCountry": "IT",
        },
        "CertIssuer": "Italian Ministry of Health",
        "CertId": "1234567890",
    }
}
