PID_DATA = {
    "eu.europa.ec.eudiw.pid.1": {
        "family_name": "Raffaello",
        "given_name": "Mascetti",
        "birth_date": "1922-03-13",
        "age_over_18": True,
        "age_over_NN": 150,
        "age_in_years": 101,
        "age_birth_year": 1922,
        "unique_id": "random-opaque-value",
        "family_name_birth": "Mascetti",
        "given_name_birth": "Raffaello",
        "birth_place": "Firenze",
        "birth_country": "IT",
        "birth_state": "Tuscany",
        "birth_city": "Firenze",
        "resident_address": "via Tal dei tali",
        "resident_country": "Italy",
        "resident_state": "Tuscany",
        "resident_city": "Firenze",
        "resident_postal_code": "50100",
        "resident_street": "via Tal dei tali",
        "resident_house_number": "21",
        "gender": 1,
        "nationality": "IT",
        "portrait": None,
        "portrait_capture_date": None,
        "issuance_date": "2023-06-07",
        "expiry_date": "2027-06-07",
        "issuing_authority": "IT",
        "document_number": "CA0568IT_example",
        "administrative_number": None,
        "issuing_country": "IT",
        "issuing_jurisdiction": None
    }
}
